import React, {Component} from 'react'
import PropTypes from 'prop-types'
import { is, fromJS } from 'immutable'
import { Button, notification, Dropdown } from 'antd'

import MKEmitter from '@/utils/events.js'
import MkIcon from '@/components/mk-icon'

import './index.scss'

class TabButton extends Component {
  static propTpyes = {
    BData: PropTypes.any,             // 主表数据
    BID: PropTypes.any,
    MenuID: PropTypes.string,         // 菜单ID
    btn: PropTypes.object,            // 按钮
    selectedData: PropTypes.any,      // 子表中选择数据
    disabled: PropTypes.any,          // 行按钮禁用
    name: PropTypes.any
  }

  state = {
    disabled: false,
    hidden: false,
    primaryId: '',
    dict: window.GLOB.dict
  }

  UNSAFE_componentWillMount () {
    const { btn, selectedData, BData, disabled } = this.props

    if (btn.controlField) {
      this.setStatus(btn, selectedData || [], BData, disabled)
    } else if (disabled) {
      this.setState({disabled: true})
    }
  }

  shouldComponentUpdate (nextProps, nextState) {
    return !is(fromJS(this.props), fromJS(nextProps)) || !is(fromJS(this.state), fromJS(nextState))
  }

  componentDidMount () {
    MKEmitter.addListener('triggerBtnId', this.actionTrigger)
  }

  UNSAFE_componentWillReceiveProps (nextProps) {
    const { btn } = this.props

    if (btn.controlField) {
      this.setStatus(btn, nextProps.selectedData || [], nextProps.BData, nextProps.disabled)
    } else {
      this.setState({disabled: nextProps.disabled === true})
    }
  }

  componentWillUnmount () {
    this.setState = () => {
      return
    }
    MKEmitter.removeListener('triggerBtnId', this.actionTrigger)
  }

  setStatus = (btn, data, BData, disprop) => {
    let disabled = false
    let hidden = false

    if (btn.control !== 'parent') {
      if (data.length > 0) {
        data.forEach(item => {
          let s = item[btn.controlField] !== undefined ? item[btn.controlField] + '' : ''
          if (btn.controlVals.includes(s) || item.$lock) {
            disabled = true
          }
        })
      } else if (btn.controlVals.includes('')) {
        disabled = true
      }
    } else {
      if (!BData || !BData.hasOwnProperty(btn.controlField)) {
        hidden = true
      } else {
        let s = BData[btn.controlField] + ''
        if (btn.controlVals.includes(s)) {
          hidden = true
        }
      }
    }

    if (disabled && btn.control === 'hidden') {
      hidden = true
    }

    if (disprop) {
      disabled = true
    }

    this.setState({hidden, disabled})
  }
  
  /**
   * @description 触发按钮操作
   */
  actionTrigger = (triggerId, record, type, lid) => {
    const { btn, selectedData, BID, LID } = this.props
    const { disabled, dict } = this.state

    if (disabled || btn.multiMenus) return
    if (triggerId && btn.uuid !== triggerId) return
    if (type === 'linkbtn' && !btn.$toolbtn && LID !== lid) return
    
    let data = record || selectedData || []

    if (btn.Ot !== 'notRequired' && data.length === 0) {
      // 需要选择行时，校验数据
      notification.warning({
        top: 92,
        message: dict['select_row'] || '请选择行！',
        duration: 5
      })
      return false
    } else if (btn.Ot === 'requiredSgl' && data.length !== 1) {
      // 需要选择单行时，校验数据
      notification.warning({
        top: 92,
        message: dict['select_single_row'] || '请选择单行数据！',
        duration: 5
      })
      return
    }

    let primaryId = ''

    if (btn.Ot === 'requiredSgl') {
      primaryId = data[0].$$uuid || ''
    } else if (btn.Ot === 'requiredOnce') {
      let ids = data.map(d => (d.$$uuid || ''))
      ids = ids.filter(Boolean)
      primaryId = ids.join(',')
    } else if (btn.Ot === 'notRequired') {
      if (btn.sysId === 'js') {
        primaryId = (() => {
          let uuid = []
          let timestamp = new Date().getTime()
          let _options = '0123456789abcdefghigklmnopqrstuv'
          for (let i = 0; i < 19; i++) {
            uuid.push(_options.substr(Math.floor(Math.random() * 0x20), 1))
          }
          uuid = timestamp + uuid.join('')
          return uuid
        })()
        primaryId = primaryId + (btn.sign || '')
      } else if (btn.sysId !== 'empty') {
        primaryId = BID || ''
      }
    }

    let newtab = {}

    if (btn.tabTemplate === 'FormTab') {
      newtab = {
        MenuNo: '',
        MenuID: btn.uuid,
        MenuName: btn.label,
        type: btn.tabTemplate,
        param: {
          parentId: this.props.MenuID,
          btn: btn,
          data: data[0] || null,
          primaryId: primaryId
        }
      }
    } else if (btn.linkmenu && btn.linkmenu.length > 0) {
      let menuId = btn.linkmenu.slice(-1)[0]
      let menu = null

      if (window.GLOB.mkThdMenus.has(menuId)) {
        menu = {...window.GLOB.mkThdMenus.get(menuId), param: { $BID: primaryId }}
      } else if (btn.MenuID) {
        menu = {
          MenuID: btn.MenuID,
          MenuName: btn.MenuName,
          type: btn.tabType,
          param: { $BID: primaryId }
        }
      }

      if (!menu) {
        notification.warning({
          top: 92,
          message: dict['no_perm'] || `没有${btn.MenuName ? `菜单《${btn.MenuName}》的` : '此菜单的'}访问权限！`,
          duration: 5
        })
        return
      }

      if (btn.Ot === 'requiredSgl' || btn.Ot === 'requiredOnce') {
        Object.keys(data[0]).forEach(key => {
          if (/^\$/.test(key)) return
          if (key === 'children') return
          menu.param[key] = data[0][key]
        })
      }

      newtab = menu
    }

    if (btn.openTab === 'view') {
      window.open('#/view/' + newtab.MenuID + '/' + primaryId)
    } else {
      MKEmitter.emit('modifyTabs', newtab, true)
  
      MKEmitter.emit('openNewTab')
    }

    if (window.GLOB.systemType === 'production') {
      MKEmitter.emit('queryTrigger', {menuId: btn.uuid, name: '标签页'})
    }
  }

  triggerMenu = (tab) => {
    const { btn, selectedData, BID } = this.props
    const { dict } = this.state

    let data = selectedData || []

    if (btn.Ot !== 'notRequired' && data.length === 0) {
      // 需要选择行时，校验数据
      notification.warning({
        top: 92,
        message: dict['select_row'] || '请选择行！',
        duration: 5
      })
      return false
    } else if (btn.Ot === 'requiredSgl' && data.length !== 1) {
      // 需要选择单行时，校验数据
      notification.warning({
        top: 92,
        message: dict['select_single_row'] || '请选择单行数据！',
        duration: 5
      })
      return
    }

    let primaryId = ''

    if (btn.Ot === 'requiredSgl') {
      primaryId = data[0].$$uuid || ''
    } else if (btn.Ot === 'requiredOnce') {
      let ids = data.map(d => (d.$$uuid || ''))
      ids = ids.filter(Boolean)
      primaryId = ids.join(',')
    } else if (btn.Ot === 'notRequired') {
      if (btn.sysId === 'js') {
        primaryId = (() => {
          let uuid = []
          let timestamp = new Date().getTime()
          let _options = '0123456789abcdefghigklmnopqrstuv'
          for (let i = 0; i < 19; i++) {
            uuid.push(_options.substr(Math.floor(Math.random() * 0x20), 1))
          }
          uuid = timestamp + uuid.join('')
          return uuid
        })()
        primaryId = primaryId + (tab.sign || '')
      } else if (btn.sysId !== 'empty') {
        primaryId = BID || ''
      }
    }

    let menuId = tab.menuId.slice(-1)[0]
    let menu = null

    if (window.GLOB.mkThdMenus.has(menuId)) {
      menu = {...window.GLOB.mkThdMenus.get(menuId), param: { $BID: primaryId }}
    } else if (tab.MenuID) {
      menu = {
        MenuID: tab.MenuID,
        MenuName: tab.MenuName,
        type: tab.tabType,
        param: { $BID: primaryId }
      }
    }

    if (!menu) {
      notification.warning({
        top: 92,
        message: dict['no_perm'] || '没有此菜单的访问权限！',
        duration: 5
      })
      return
    }


    if (btn.openTab === 'view') {
      window.open('#/view/' + menu.MenuID + '/' + primaryId)
    } else {
      MKEmitter.emit('modifyTabs', menu, true)
  
      MKEmitter.emit('openNewTab')
    }

    if (window.GLOB.systemType === 'production') {
      MKEmitter.emit('queryTrigger', {menuId: btn.uuid, name: '标签页'})
    }
  }

  render() {
    const { btn, name } = this.props
    const { disabled, hidden } = this.state

    if (hidden) return null

    let label = ''

    if (btn.show === 'link') {
      label = <span>{name || btn.label}{btn.icon ? <MkIcon style={{marginLeft: '8px'}} type={btn.icon} /> : ''}</span>
    } else if (btn.show === 'icon') {
      label = <MkIcon type={btn.icon} />
    } else {
      label = <span>{btn.icon ? <MkIcon style={{marginRight: '8px'}} type={btn.icon} /> : ''}{name || btn.label}</span>
    }

    if (btn.multiMenus && !disabled) {
      return (<Dropdown
        overlay={<div className="mk-tab-dropdown-wrap">{
          btn.multiMenus.map(tab => <div key={tab.uuid} onClick={() => this.triggerMenu(tab)}>{tab.name}</div>)
        }</div>}
        trigger={['hover']}
      >
        <Button
          type="link"
          title={btn.show === 'icon' ? btn.label : ''}
          style={btn.style || null}
          className={btn.hover || ''}
        >{label}</Button>
      </Dropdown>)
    }

    return (
      <Button
        type="link"
        id={'button' + btn.uuid}
        title={disabled ? (btn.reason || '') : (btn.show === 'icon' ? btn.label : '')}
        style={btn.style || null}
        disabled={disabled}
        className={btn.hover || ''}
        onClick={(e) => {e.stopPropagation(); this.actionTrigger()}}
      >{label}</Button>
    )
  }
}

export default TabButton